/*
 * mbox_derive_from.c
 * Get us a mbox-style From_ header
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_derive_from.c,v 1.12 2001/10/23 10:17:33 fullermd Exp $
 */

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>


void
mbox_derive_from(char **hdr_in)
{
	char *buf, *buf2, *buf3;
	char *from_hdr=NULL, *recip;
	char *intbuf;
	char *ctret;
	int sender=0;
	/* Note that order in these are important */
	char *from_good[]=
	{
		"X-From_:"
	};
	char *from_possible[]=
	{
		"Return_Path:",
		"Sender:",
		"From:",
	};
	/* XXX Keep these in sync with changes to above */
	int fgmax=0, fpmax=2;
	int i, found;
	time_t rtime;

	intbuf = malloc(strlen(*hdr_in)+1);
	strcpy(intbuf, *hdr_in);
	buf = intbuf;

	/* First, see if we can save a lot of trouble */
	if(!(strncmp(buf, "From ", strlen("From "))))
	{
		/* We already have a From_ header */
		free(intbuf);
		return;
	}

	/* 
	 * Ordering considerations:
	 * I've decided, unilaterally, in a undemocratic way,
	 * the order in which we'll search.
	 * Justifications:
	 * X-From_ - some POP servers (cucipop for ex.) seem to add it
	 * Return-Path || Sender || From: - mutt uses this order
	 * Note that those latters need some frobbing...
	 * Ordering is done according to above defined arrays.
	 */
	found=0;
	for(i=0 ; ((i<=fgmax)&&(!found)) ; i++)
	{
		if( (buf3=strstr(buf, from_good[i])) != NULL )
		{
			/* Check if we're at beginning of line */
			if( ((buf3==buf) || (*(buf3-1) == '\n')) )
			{
				/* We have a winnah! */
				found=1;

				if( !(buf2=strstr(buf3, "\r\n")) )
					buf2=strchr(buf3, '\n');

				if(!buf2)
				{
					/* Hm.  This shouldn't happen */
					free(intbuf);
					return;
				}

				*buf2 = '\0';

				buf3 = strchr(buf3, ':');
				buf3++;
				while(*buf3==' ')
					buf3++;
				from_hdr = malloc(strlen(buf3)+6+1);
				strcpy(from_hdr, buf3);
			}
		}
	}
	for(i=0 ; ((i<=fpmax)&&(!found)) ; i++)
	{
		if( (buf3=strstr(buf, from_possible[i])) != NULL )
		{
			/* Check if we're at beginning of line */
			if( ((buf3==buf) || (*(buf3-1) == '\n')) )
			{
				/* We have a winnah! */
				found=1;

				if( !(buf2=strstr(buf3, "\r\n")) )
					buf2=strchr(buf3, '\n');

				if(!buf2)
				{
					/* Hm.  This shouldn't happen */
					free(intbuf);
					return;
				}

				*buf2 = '\0';

				/*
				 * Now, this is where it gets ugly.
				 * For these headers, we want to extract
				 * the sender from within <>'s, and then append
				 * the current date/time.
				 */
				recip = strchr(buf3, '<');
				if(!recip)
				{
					/*
					 * Unfortunately, "Sender:" doesn't
					 * use RFC822 <>-form.
					 * Sigh.  Dammit.  If it's not one thing,
					 * it's your mother.
					 */
					if( !(strncmp("Sender: ", buf3, 8)) )
					{
						/*
						 * Yup, it's a sender header.
						 * We're going to be really hackish.
						 * Somebody find a better way to do this.
						 */
						sender=1;
						/* XXX Only +7 because we ++ below */
						recip = buf3 + 7;
					}
					else
					{
						/*
						 * Ugh.  Problem.  Malformed header.
						 * Danger, Will Robinson!
						 */
						free(intbuf);
						return;
					}
				}

				recip++;

				buf2=strchr(recip, '>');
				if(!buf2 && !sender)
				{
					/*
					 * Ugh.  Problem.  Malformed header.
					 * Danger, Will Robinson!
					 */
					free(intbuf);
					return;
				}

				if(buf2)
					*buf2='\0';
				rtime = time(NULL);
				ctret = ctime(&rtime);
				/* +1 for space, +1 for \0 */
				from_hdr = malloc(strlen(recip)+strlen(ctret)+1+1);
				sprintf(from_hdr, "%s %s", recip, ctret);
				free(ctret);
			}
		}
	}

	free(intbuf);

	/* Don't forget the extra +6 for "From " and \n */
	intbuf=malloc( strlen(from_hdr) + strlen(*hdr_in) + 6 + 1 );
	sprintf(intbuf, "From %s\n%s", from_hdr, *hdr_in);
	free(*hdr_in);
	*hdr_in = malloc(strlen(intbuf)+1);
	strcpy(*hdr_in, intbuf);
	free(intbuf);
	free(from_hdr);
	return;
}
