/*
 * mbox_get_message.c
 * Get (parts of) a specific message from a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_get_message.c,v 1.5 2001/12/25 16:57:11 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/uio.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>

/* 128k.  This can be tuned, and should be made tunable. */
#define READ_CHUNK	131072


int
mbox_get_message(POSTAL_CONN *p_conn, POSTAL_MSG *msg, int get_type)
{
	char *rawmsg, *rawloc;
	char *buf;
	int gotmsg=0;
	ssize_t readlen;
	size_t bufavail, bufsz;

	/* Get to where we 'think' we're starting */
	lseek(p_conn->spec.mbox->desc, msg->loc.offset, SEEK_SET);

	/*
	 * XXX NOTE!
	 * We assume that nothing has changed since these
	 * messages were pulled out of the mailbox!
	 * Fixme later.
	 */

	/* We're going to use READ_CHUNK as our base unit */
	rawmsg = malloc(READ_CHUNK);
	rawloc = rawmsg;
	bufavail = READ_CHUNK;
	bufsz = READ_CHUNK;

	while(gotmsg==0)
	{
		/* Read in a block */
		if( (readlen = read(p_conn->spec.mbox->desc, rawloc, (bufavail-1)))
				> 0)
		{
			rawloc[readlen] = '\0';
			bufavail -= readlen;
		}

		if( (readlen==0) && (bufavail==bufsz) )
		{
			/*
			 * No message to read, we're starting at EOF.
			 * This shouldn't happen.
			 */
			free(rawmsg);
			return(-1);
		}

		/*
		 * See if we have the beginning of the next message.
		 * If we do, cut off right before it, adjust sizing,
		 * and all that jazz.
		 * If we don't, get read to read() more.
		 */
		if( (buf=strstr(rawmsg, "\n\nFrom ")) || readlen==0)
		{
			/*
			 * We've either got a full message with the
			 * beginning of the next, or we're at EOF.
			 */
			if( readlen != 0 )
			{
				/*
				 * We've got a full message with one starting after.
				 * Record the extra length and lseek() back to the
				 * beginning of the next message.
				 */
				buf += 2;
				*buf = '\0';
				lseek(p_conn->spec.mbox->desc,
						(msg->loc.offset + strlen(rawmsg)), SEEK_SET);
			}

			/* Find the end of the headers */
			buf = strstr(rawmsg, "\n\n");
			*buf = '\0';
			buf += 2;

			/* Shovel the data into the struct */
			if( (get_type==GET_HEADER) || (get_type == GET_BODY) )
			{
				msg->header = malloc(strlen(rawmsg)+1);
				strcpy(msg->header, rawmsg);
			}

			if( (get_type==GET_BODY) )
			{
				msg->body = malloc(strlen(buf)+1);
				strcpy(msg->body, buf);
			}

			gotmsg=1;
		}
		else
		{
			/* Haven't gotten it yet, let's push onward */
			/* Alloc more */
			rawmsg = realloc(rawmsg, (bufsz + READ_CHUNK));
			bufsz += READ_CHUNK;
			bufavail += READ_CHUNK;
			rawloc = rawmsg + (bufsz - bufavail);

			/* Loop around again */
		}
	}

	/* We're all filled in and it worked fine.  Return */
	return(0);
}
