/*
 * mbox_get_message_next.c
 * Get the 'next' message from a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_get_message_next.c,v 1.7 2001/12/25 16:57:11 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/uio.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>

/* 128k.  This can be tuned, and should be made tunable. */
#define READ_CHUNK	131072


POSTAL_MSG *
mbox_get_message_next(POSTAL_CONN *p_conn, int get_type)
{
	POSTAL_MSG *toret;
	char *rawmsg, *rawloc;
	char *buf;
	int gotmsg=0;
	ssize_t readlen;
	size_t bufavail, bufsz;

	toret = malloc(PMSG_SIZE);

	/*
	 * First record where we're "starting".
	 * We're going to assume that we're at the beginning of
	 * a message within the mbox.
	 * May need to revisit this later and do more error-checking.
	 */
	toret->loc.offset = lseek(p_conn->spec.mbox->desc, 0, SEEK_CUR);

	/* We're going to use READ_CHUNK as our base unit */
	rawmsg = malloc(READ_CHUNK);
	rawloc = rawmsg;
	bufavail = READ_CHUNK;
	bufsz = READ_CHUNK;

	while(gotmsg==0)
	{
		/* Read in a block */
		if( (readlen = read(p_conn->spec.mbox->desc, rawloc, (bufavail-1)))
				> 0)
		{
			rawloc[readlen] = '\0';
			bufavail -= readlen;
		}

		if( (readlen==0) && (bufavail==bufsz) )
		{
			/* No message to read, we're starting at EOF */
			free(rawmsg);
			return(NULL);
		}

		/*
		 * See if we have the beginning of the next message.
		 * If we do, cut off right before it, adjust sizing,
		 * and all that jazz.
		 * If we don't, get read to read() more.
		 */
		if( (buf=strstr(rawmsg, "\n\nFrom ")) || readlen==0)
		{
			/*
			 * We've either got a full message with the
			 * beginning of the next, or we're at EOF.
			 */
			if( readlen != 0 )
			{
				/*
				 * We've got a full message with one starting after.
				 * Record the extra length and lseek() back to the
				 * beginning of the next message.
				 */
				buf += 2;
				*buf = '\0';
				lseek(p_conn->spec.mbox->desc,
						(toret->loc.offset + strlen(rawmsg)), SEEK_SET);
			}

			/* Note the full in-file size */
			toret->msglen = strlen(rawmsg);

			/* Find the end of the headers */
			buf = strstr(rawmsg, "\n\n");
			*buf = '\0';
			buf += 2;

			/* Shovel the data into the struct */
			if( (get_type==GET_HEADER) || (get_type == GET_BODY) )
			{
				toret->header = malloc(strlen(rawmsg)+1);
				strcpy(toret->header, rawmsg);
			}

			if( (get_type==GET_BODY) )
			{
				toret->body = malloc(strlen(buf)+1);
				strcpy(toret->body, buf);
			}

			gotmsg=1;
		}
		else
		{
			/* Haven't gotten it yet, let's push onward */
			/* Alloc more */
			rawmsg = realloc(rawmsg, (bufsz + READ_CHUNK));
			bufsz += READ_CHUNK;
			bufavail += READ_CHUNK;
			rawloc = rawmsg + (bufsz - bufavail);

			/* Loop around again */
		}
	}

	/*
	 * We've got the message now, parsed out
	 * and everything.
	 * Cleanup and return.
	 */
	free(rawmsg);

	return(toret);
}
